"======================================================================
"cream-spell.vim
"
" Cream -- An easy-to-use configuration of the famous Vim text editor
" [ http://cream.sourceforge.net ] Copyright (C) 2002-2004  Steve Hall
" 
" License:
" This program is free software; you can redistribute it and/or modify
" it under the terms of the GNU General Public License as published by
" the Free Software Foundation; either version 2 of the License, or
" (at your option) any later version.
" [ http://www.gnu.org/licenses/gpl.html ]
" 
" This program is distributed in the hope that it will be useful, but
" WITHOUT ANY WARRANTY; without even the implied warranty of
" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
" General Public License for more details.
" 
" You should have received a copy of the GNU General Public License
" along with this program; if not, write to the Free Software
" Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
" 02111-1307, USA.
"

" Description: {{{1
"
" Origins:
" This file originated from Dr. Charles E. Campbell, Jr.'s wonderful
" spell checking work for Vim, available all over the web, but
" specifically available from:
"   http://vim.sourceforge.net/scripts/script.php?script_id=195
"
" At this point, modifications include many essential changes to work
" with Cream and from insertmode, as well as many other major and
" minor adjustments that we've integrated over time. The original
" engine still lives in function Cream_spell(), but with so many
" changes Dr. Chip might not even recognize it.
"
" However we strongly suggest investigating Campbell's work, since it
" is expert and well-maintained. This project will attempt to include
" those necessary improvements from the original, but we recognize
" that it will be somewhat difficult given the original author's
" expertise and our own project's special interests.
"
" Notes:
" * g:CREAM_SPELL_LANG is an array of selected dictionaries:
"       "en,s,US,CA,BR;de,large,AU"
"   where:
"        xx,               <-- language code (specified by ISO 639)
"           x,             <-- dictionary size (s,m,l,x)
"             xx,          <-- dialect (country) code (two digits)
"                xx,       <--           "
"                   xx     <--           "
"                     ;    <-- semi-colon language separator
"
" * Names of available dictionaries would then be in the format:
"   cream-spell-dict-en-s-US_1.vim -- case insensitive
"                           _2     -- case sensitive
"                       l-CA_3     -- case insensitive, multi-word
"                       l-CA_4     -- case sensitive, multi-word
"
" * g:CREAM_SPELL_MULTIDICT holds the option value to use the default
"   dictionary(0), a single language dictionary(1) or multiple
"   dictionaries(2) at time.
"
" * Test words:
"   - English test word: correct
"   - French test word:  assemblais assembleuse assemblée fidéjussion
"   - German test word:  abschweifendem
"   - Spanish test word: acurrucarive
"
" *** Note: highlighting is/must be defined elsewhere ***
" *** (cream-colors.vim) so as to be themeable        ***

" Initialization {{{1

function! Cream_spell_init()
	" spell check options (if don't exist)
	if !exists("g:CREAM_SPELL_MULTIDICT")
		" default no ('0', '1' = Single, '2' = Multiple)
		let g:CREAM_SPELL_MULTIDICT = 0
	endif

	" language (list of sourced dictionaries)
	" * default is empty ("")
	" * if g:CREAM_SPELL_MULTIDICT == 2, allowed to be multiple
	if !exists("g:CREAM_SPELL_LANG") || g:CREAM_SPELL_MULTIDICT == 0
		let g:CREAM_SPELL_LANG = ""
	endif

	" set spell check dictionary path
	let g:cream_spelldicts = $CREAM . "spelldicts/"

endfunction
" now called via autocmd
"call Cream_spell_init()

" Functions called by mappings {{{1

" toggle error highlighting on/off
function! Cream_spellcheck()
	if !exists("b:cream_spell")
		" set
		let b:cream_spell = "1"
		" load main function
		call Cream_spell()
        "" refresh colors
        "call Cream_colors()
	else
		" unset
		unlet b:cream_spell
        " clear sets
		silent! syntax clear BadWord
		silent! syntax clear GoodWord
		" reset original highlighting
		call Cream_filetype()
		" refresh color theme
		call Cream_colors()
	endif
endfunction

"" reinitialize syntax highlighting on enter
"function! Cream_spellcheck_set_enter()
"    if exists("b:cream_spell")
"        
"        "***
"        " magical refresh syntax highlighting w/out re-reading file here
"        "***

"        "" trick nosyntax.vim ;>
"        "unlet! syntax
"        "" load main function
"        "call Cream_spell()
"        "" refresh colors
"        "call Cream_colors()
"    endif
"endfunction
"autocmd BufEnter * call Cream_spellcheck_set_enter()

" Next word (and turn on if not on)
function! Cream_spell_next()
	" if highlighting not already on, turn on
	if !exists("b:cream_spell")
		call Cream_spellcheck()
	endif
	call Cream_SpchkNxt()
endfunction

" Previous word (and turn on if not on)
function! Cream_spell_prev()
	" if highlighting not already on, turn on
	if !exists("b:cream_spell")
		call Cream_spellcheck()
	endif
	call Cream_SpchkPrv()
endfunction

" Save word to dictionary
function! Cream_spell_saveword()
	" only perform if spell check turned on
	if exists("b:cream_spell")
		let myword = expand("<cword>")
		call Cream_spell_saveword_dialog(myword)
	else
		call confirm("Please turn on spell check before adding words to dictionary.", "&Ok", 1, "Warning")
	endif
endfunction

function! Cream_spell_saveword_v()

	" assign selection (on register "x") to variable
	let myword = @x

	" only perform if spell check turned on
	if exists("b:cream_spell")
		call Cream_spell_saveword_dialog(myword)
	else
		call confirm("Please turn on spell check before adding words to dictionary.", "&Ok", 1, "Warning")
	endif

	" reselect
	normal gv

endfunction
function! Cream_spell_saveword_dialog(myword)
	" allow editing
	let myreturn = "bad"
	while myreturn == "bad"
		let myword = inputdialog("Verify word to be saved:", a:myword)
		if myword != ""
			" check that only legal characters are being used
			" * highlighting keywords: transparent contained contains
			"   display extend fold skip
			let mymatch = ""
			if match(g:CREAM_SPELL_LANG, "dut") != -1
				" from Thijs Max <n.t.p.max (at) student.tudelft.nl> 2004-01-27
				if &encoding == "latin1"
					let mymatch = mymatch . "[a-zA-ZƒŸµÀÁÂÄÆÇÈÉÊËÌÍÎÏÒÓÔÖÙÚÛÜÝÞßàáâäæçèéêëìíîïòóôöùúûüýþÿ0-9&-_]"
				endif
			endif
			if match(g:CREAM_SPELL_LANG, "eng") != -1
				let mymatch = mymatch . "[a-zA-Z0-9&-_]"
			endif
			if match(g:CREAM_SPELL_LANG, "fre") != -1
				" from Christian Goebel <goebbe (at) yahoo.com> 2002-11-23
				if &encoding == "latin1"
					let mymatch = mymatch . "[a-zA-ZÁáÂâÄäÇçÈèÉéÊêËëÎîÏïÔôÖöÛûÀà0-9&-_]"
				endif
			endif
			if match(g:CREAM_SPELL_LANG, "ger") != -1
				if &encoding == "latin1"
					let mymatch = mymatch . "[a-zA-ZäÄüÜöÖß0-9&-_]"
				endif
			endif
			if match(g:CREAM_SPELL_LANG, "spa") != -1
				" from Jose Alberto Suarez Lopez <bass (at) gentoo.org>
				if &encoding == "latin1"
					let mymatch = mymatch . "[a-zA-ZÁáÉéÍíÑñÓóÚúÜü0-9&-_]"
				endif
			endif
			if match(myword, "\@!" . mymatch) != -1
				call confirm("Sorry, only letters, numbers, spaces, \"&\", \"-\", \"_\" allowed.", "&Ok", 1, "Info")
			else
				call Cream_SpchkSave(myword)
				break
			endif
		else
			break
		endif
	endwhile
endfunction

"*** Fails! Module uses the external agrep
"" Show alternate words
"imap <silent> <C-F7> <C-o>:call Cream_spell_altword()<CR>
"function! Cream_spell_altword()
"	" only perform if spell check turned on
"	if exists("b:cream_spell")
"		let myword = expand("<cword>")
"		execute ":call SpchkAlternate(myword)"
"	endif
"endfunction
"***


" Cream_spell() {{{1
function! Cream_spell()
" *** LOAD ON DEMAND! Too slow to load on startup ***

	"----------------------------------------------------------------------
	" establish BadWord

	" The Raison D'Etre!
	" BadWord (misspelling) highlighting done here.  Plus comment
	" support -- do spell-checking inside comments. This can be done
	" only for those syntax files' comment blocks that
	" contains=@cluster. The code below adds GoodWord and BadWord to
	" various clusters.  If your favorite syntax comments are not
	" included, send a note to your syntax file's maintainer about it!
	if     &filetype == "amiga"
		syntax cluster amiCommentGroup add=GoodWord,BadWord
		syntax match BadWord contained "\<\a\+\>"
	elseif &filetype == "bib"
		syntax cluster bibVarContents contains=GoodWord,BadWord
		syntax cluster bibCommentContents contains=GoodWord,BadWord
		syntax match BadWord contained "\<\a\+\>"
	elseif &filetype == "c" || &filetype == "cpp"
		syntax cluster cCommentGroup add=GoodWord,BadWord
		syntax match BadWord contained "\<\a\+\>"
	elseif &filetype == "csh"
		syntax cluster cshCommentGroup add=GoodWord,BadWord
		syntax match   BadWord contained "\<\a\+\>"
	elseif &filetype == "dcl"
		syntax cluster dclCommentGroup add=GoodWord,BadWord
		syntax match BadWord contained "\<\a\+\>"
	elseif &filetype == "fortran"
		syntax cluster fortranCommentGroup	add=GoodWord,BadWord
		syntax match BadWord contained "\<\a\+\>"
		syntax match fortranGoodWord contained	"^[Cc]\>"
		syntax cluster fortranCommentGroup add=fortranGoodWord
		highlight link fortranGoodWord fortranComment
	elseif &filetype == "sh" || &filetype == "ksh" || &filetype == "bash"
		syntax cluster shCommentGroup add=GoodWord,BadWord
		syntax match BadWord contained "\<\a\+\>"
	elseif &filetype == "tex"
		syntax match GoodWord "{[a-zA-Z|@]\+}"lc=1,me=e-1
		syntax match GoodWord "\[[a-zA-Z]\+]"lc=1,me=e-1
		syntax match texGoodWord "\\[a-zA-Z]\+"lc=1,me=e-1 contained
		highlight link texGoodWord texComment
		syntax cluster texCommentGroup add=texGoodWord
	elseif &filetype == "vim"
		syntax cluster vimCommentGroup add=GoodWord,BadWord
		syntax match BadWord contained "\<\a\+\>"
	else
		" highlight all words as BadWords (redeem below)
		syntax match BadWord "\<\a\+\>"
	endif

	" remainder are in BadWord
	if match(g:CREAM_SPELL_LANG, "dut") != -1
		if &encoding == "latin1"
			syntax match BadWord "\<[a-zA-ZƒŸµÀÁÂÄÆÇÈÉÊËÌÍÎÏÒÓÔÖÙÚÛÜÝÞßàáâäæçèéêëìíîïòóôöùúûüýþÿ0-9&\-_]\+\>"
		else
			syntax match BadWord "\<[a-zA-Z0-9&\-_]\+\>"
		endif
	endif
	if match(g:CREAM_SPELL_LANG, "eng") != -1
		syntax match BadWord "\<[a-zA-Z0-9&\-_]\+\>"
	endif
	if match(g:CREAM_SPELL_LANG, "fre") != -1
		if &encoding == "latin1"
			syntax match BadWord "\<[a-zA-ZÁáÂâÄäÇçÈèÉéÊêËëÎîÏïÔôÖöÛûÀà0-9&\-_]\+\>"
		else
			syntax match BadWord "\<[a-zA-Z0-9&\-_]\+\>"
		endif
	endif
	if match(g:CREAM_SPELL_LANG, "ger") != -1
		if &encoding == "latin1"
			syntax match BadWord "\<[a-zA-ZäÄüÜöÖß0-9&\-_]\+\>"
		else
			syntax match BadWord "\<[a-zA-Z0-9&\-_]\+\>"
		endif
	endif
	if match(g:CREAM_SPELL_LANG, "spa") != -1
		if &encoding == "latin1"
			syntax match BadWord "\<[a-zA-ZÁáÉéÍíÑñÓóÚúÜü0-9&\-_]\+\>"
		else
			syntax match BadWord "\<[a-zA-Z0-9&\-_]\+\>"
		endif
	endif


	"----------------------------------------------------------------------
	" establish GoodWord
	" Note: *matches* need to follow the BadWord so that they take
	" priority!

	" load dictionaries
	" -----------------
	call Cream_spell_dictload()


	" case sensitive
	" --------------
	syntax case match

	" ignores people's middle-name initials
	syntax match GoodWord "\<[A-Z]\."


	" case insensitive
	" ----------------
	syntax case ignore

	""" allows <engspchk.vim> to work better with LaTeX
	""if &filetype == "tex"
	""    syntax match GoodWord "{[a-zA-Z|@]\+}"lc=1,me=e-1
	""    syntax match GoodWord "\[[a-zA-Z]\+]"lc=1,me=e-1
	""    syntax match texGoodWord "\\[a-zA-Z]\+"lc=1,me=e-1 contained
	""    highlight link texGoodWord texComment
	""    syntax cluster texCommentGroup add=texGoodWord
	""endif

	" Abbreviations, Possessives, Etc.  For these to be recognized
	" properly, these contractions' word prior to the "'" has been
	" removed from the keyword dictionaries and moved here.
	" avoids a lot of useless matches!
	if match(g:CREAM_SPELL_LANG, "dut") != -1
		syntax match GoodWord "'s\>"
		syntax match GoodWord "s'\>"
		syntax match GoodWord "ex-"
	endif
	if match(g:CREAM_SPELL_LANG, "eng") != -1
		syntax match GoodWord "'s\>"
		syntax match GoodWord "ex-"
	endif
	if match(g:CREAM_SPELL_LANG, "ger") != -1
		syntax match GoodWord "'s\>"
		syntax match GoodWord "ex-"
	endif


	" exist in proper match dictionaries now
	"" these are words but have special meaning in vim files, so they can't be keywords :o
	"syntax match GoodWord "\<\(transparent\|contained\|contains\|display\|extend\|fold\|skip\)\>"

	" ignore web addresses and \n for newlines (case insensitive)
	syntax match GoodWord "\<http://www\.\S\+"
	syntax match GoodWord "\\n"

	" Chase Tingley patch: prevents Error highlighting of words while
	" one is typing them.  \%* is a new magic atom that matches
	" zero-length if that is where the cursor currently is.
	syntax match GoodWord "\<\a\+\%#\>"
	syntax match GoodWord "\<\a\+'\%#"

	" numbers
	syntax match GoodWord "\<\d\+\>"
	" TEST: 8848

	" mixed numbers and letters
	" TEST: 8848a8 a8848 8848a
	if  match(g:CREAM_SPELL_LANG, "eng") != -1
		" preceding alpha
		syntax match GoodWord "\<\a\+\d\+\>"
		" trailing alpha
		syntax match GoodWord "\<\d\+\a\+\>"
		" embedded alpha
		syntax match GoodWord "\<\d\+\a\+\d\+\>"
		"" combined (broken)
		"syntax match GoodWord "\<\a\*\d\+\a\*\d\*\>"
	endif

	"----------------------------------------------------------------------
	" catch multi- or cross-word errors

	" ellipses
	"
	" TEST PRECINCT:
	" cow. bells
	" cow. Bells
	" cow.  bells
	" cow.  Bells
	" cow.   bells
	" cow.   Bells
	"
	" cow...bells
	" cow... bells
	" cow...  bells
	"
	" cow....bells
	" cow....Bells
	" cow.... bells
	" cow.... Bells
	" cow....  bells
	" cow....  Bells
	" END TEST PRECINCT:
	"
	" non-capitalized word after ellipses...
	syntax match GoodWord "\.\.\. \{0,2}\l\@="
	" ...but not non-capitalized word after ellipses plus period
	syntax match BadWord "\.\.\.\. \{0,2}\l"


	" non-lowercased end of words (in order, as required)
	" * required: period/question mark/exclamation mark
	" * optional: end-quotes (allow nested)
	" * required: returns/return-linefeeds/tabs/spaces
	" * optional: begin-quote (allow nested)
	" * optional: begin punctuation
	" * required: lowercase letter
	"
	" TEST PRECINCT:
	" cow. "bells
	" cow. 'bells
	" cow. `bells
	" cow. «bells
	" END TEST PRECINCT:
	"

	" Dutch (Antoine J. Mechelynck, 2002-08-03)
	if match(g:CREAM_SPELL_LANG, "dut") != -1
		if &encoding == "latin1"
			syntax match BadWord "[\.?!][\"']*[\r\n\t ]\+[\"'`]*\l"
		endif
	endif
	" English (Steve Hall)
	if match(g:CREAM_SPELL_LANG, "eng") != -1
		syntax match BadWord "[\.?!][\"']*[\r\n\t ]\+[\"'`]*\l"
	endif
	" French ()
	if match(g:CREAM_SPELL_LANG, "fre") != -1
		syntax match BadWord "[\.?!][\"']*[\r\n\t ]\+[\"'`]*\l"
	endif
	" German (Wolfgang Hommel, 2002-11-23)
	if match(g:CREAM_SPELL_LANG, "ger") != -1
		if &encoding == "latin1"
			syntax match BadWord "[\.?!][\"'«]*[\r\n\t ]\+[\"'»]*\l"
		endif
	endif
	" Spanish ()
	if match(g:CREAM_SPELL_LANG, "spa") != -1
		if &encoding == "latin1"
			syntax match BadWord "[\.?!][\"']*[\r\n\t ]\+[\"'`]*[¡¿]\=\l"
		endif
	endif


	" double word checking (TEST: the the coww )
	if  match(g:CREAM_SPELL_LANG, "dut") != -1
	\|| match(g:CREAM_SPELL_LANG, "eng") != -1
	\|| match(g:CREAM_SPELL_LANG, "fre") != -1
	\|| match(g:CREAM_SPELL_LANG, "ger") != -1
	\|| match(g:CREAM_SPELL_LANG, "spa") != -1
		" Bram Moolenaar:
		"syntax match DoubleWord "\(\<\k\+\>\)\s\+\1"
		" Preben Peppe Guldberg:
		"syntax match DoubleWord "\(\<\k\+\>\)\s\+\1\>"
		" Gary Holloway:
		"syntax match DoubleWord "\(\<\k\+\>\)\(\_s\+\<\1\>\)\+"
		" Rober Montante:
		"syntax match DoubleWord "\(\<\w\+\>\)\_s*\1\>"
		" Benji Fisher:
		"syntax match DoubleWord "\<\(\k\+\)\_s\+\1\>"
		" Chip Campbell: (ahead of keyword recognition)
		"syntax match DoubleWord "\s\(\k\+\)\_s\+\1\>"ms=s+1
		" Testing:
		"syntax match DoubleWord "\(\<\k\+\>\)\_s\+\1"
	endif

endfunction

" Cream_spell_dictload() {{{1
function! Cream_spell_dictload()
" load spell check dictionaries based on user preferences (set elsewhere)

	" user (previously discovered and validated in creamrc)
	syntax case ignore
	call Cream_source(g:cream_user . "spelldicts/cream-spell-dict-usr_1.vim")
	call Cream_source(g:cream_user . "spelldicts/cream-spell-dict-usr_3.vim")
	syntax case match
	call Cream_source(g:cream_user . "spelldicts/cream-spell-dict-usr_2.vim")
	call Cream_source(g:cream_user . "spelldicts/cream-spell-dict-usr_4.vim")

	" languages (read above for explanation of g:CREAM_SPELL_LANG array)
	let i = 0
	" for each language
	let max = MvNumberOfElements(g:CREAM_SPELL_LANG, ";")
	while i <= max
		" get entire language element
		let mylangstr = MvElementAt(g:CREAM_SPELL_LANG, ";", i)
		" read language item from it (item 0)
		let mylang = MvElementAt(mylangstr, ",", 0)

		" if empty, languages empty
		if mylang == "" && i == 0
			" load default and quit
			call Cream_spell_dictload_default()
			return
		endif

""*** DEBUG:
"call confirm(
"	\"DEBUG: in Cream_spell_dictload() \n" .
"	\ " mylang    = " . mylang . "\n" .
"	\ " mylangstr = " . mylangstr . "\n" .
"	\ "\n", "&Ok", 1, "Info")
""***
		" load dictionaries (MULTIPLE ALLOWED, DON'T MAKE THESE EITHER/OR!)

		" Dutch
		if mylang == "dut"
			call Cream_spell_dictload_lang(mylangstr)
		endif
		" English [default]
		if mylang == "eng"
			call Cream_spell_dictload_lang(mylangstr)
		endif
		" French
		if mylang == "fre"
			call Cream_spell_dictload_lang(mylangstr)
		endif
		" German
		if mylang == "ger"
			call Cream_spell_dictload_lang(mylangstr)
		endif
		" Spanish
		if mylang == "spa"
			call Cream_spell_dictload_lang(mylangstr)
		endif

		let i = i + 1

	endwhile

endfunction

" Cream_spell_dictload_default() {{{1
function! Cream_spell_dictload_default()
" default dictionary loading (loaded these if no language selected)
	let n = 0
	syntax case ignore
	let n = n + Cream_source(g:cream_spelldicts . "cream-spell-dict-eng-s_1.vim")
	let n = n + Cream_source(g:cream_spelldicts . "cream-spell-dict-eng-s_3.vim")
	syntax case match
	let n = n + Cream_source(g:cream_spelldicts . "cream-spell-dict-eng-s_2.vim")
	let n = n + Cream_source(g:cream_spelldicts . "cream-spell-dict-eng-s_4.vim")
	if n != 4
		call confirm("Warning, default dictionary not found.", "&Ok", 1, "Warning")
	endif
endfunction

" Cream_spell_dictload_lang() {{{1
function! Cream_spell_dictload_lang(lang)
" load English dictionaries (unique for this language)
" * a:lang should be the entire language's string!

	" get dict size
	let mydictsize = MvElementAt(a:lang, ",", 1)
	" add all that are smaller (dictionaries are cumulative)
	if     mydictsize == ""
		" we need an element to iterate once if empty
		let mydictsize = "nosize"
	elseif mydictsize == "s"
		" we're okey dokey
	elseif mydictsize == "m"
		let mydictsize = "s,m,"
	elseif mydictsize == "l"
		let mydictsize = "s,m,l,"
	elseif mydictsize == "x"
		let mydictsize = "s,m,l,x,"
	else
		call confirm("Error determining sizes in Cream_spell_dictload_lang()", "&Ok", 1)
	endif

	" get dialects (items remaining)
	let j = 2
	let mydialects = ""
	let max = MvNumberOfElements(a:lang, ",")
	while j < max
		let myelement = MvElementAt(a:lang, ",", j)
		"if mydialects != 0
		"	call confirm("Error: no dialect discovered in g:CREAM_SPELL_LANG, eng element", "&Ok", 1, "Info")
		"	return
		"endif
		let mydialects = MvAddElement(mydialects, ",", myelement)
		let j = j + 1
	endwhile

""*** DEBUG:
"call confirm(
"	\"DEBUG: in Cream_spell_dictload_lang() \n" .
"	\" a:lang        = " . a:lang . "\n" .
"	\" mydictsize    = " . mydictsize . "\n" .
"	\" mydialects    = " . mydialects . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***

	" compose list of all dictionaries
	let loadedfiles = ""
	let filesizes = 1	" avoid divide by 0 ;)
	" sizes
	let i = 0
	let max = MvNumberOfElements(mydictsize, ",")
	while i < max

		" dialects (-1 includes default/dialect-free first)
		let j = -1
		let max2 = MvNumberOfElements(mydialects, ",")
		while j < max2

			" create file name (minus part indicator)
			" language
			let myfile = g:cream_spelldicts . "cream-spell-dict-" . MvElementAt(a:lang, ",", 0)

			" append size (if present and not empty)
			if mydictsize != "" && mydictsize != "nosize"
				let myfile = myfile . "-" . MvElementAt(mydictsize, ",", i)
			endif

			" append dialects (if exist)
			if j == -1
				" do base language (dialect-free) first (ignore dialect extension)
			else
				" add dialects (if exist)
				let mydialect_ = MvElementAt(mydialects, ",", j)
				if MvNumberOfElements(mydialects, ",") > 0 && mydialect_ != ""
					let myfile = myfile . "-" . mydialect_
				endif
			endif

			" append 4 part names
			let k = 1		" 4 dict parts (upper | lower && single | multiword)
			while k <= 4

				" append individual number and extension
				let tempfile = myfile . "_" . k . ".vim"

				" total file sizes
				let filesizes = filesizes + getfsize(tempfile)

				" add to list
				let loadedfiles = MvAddElement(loadedfiles, "\n", tempfile)

				" add 1 for all, 2 to omit case-sensitive (not slow, don't bother)
				let k = k + 1

			endwhile
			let j = j + 1

		endwhile
		let i = i + 1

	endwhile

	" load list
	let myfilesizepart = ""
	let i = 0
	let max = MvNumberOfElements(loadedfiles, "\n")
	while i < max

		" get file
		let myfile = MvElementAt(loadedfiles, "\n", i)

		" show progress bar
		" get filesize
		let myfilesize = getfsize(myfile)
		" calculate filesize percentage of total
		let myfilesizeparttmp = (myfilesize * 100) / filesizes
		let myfilesizepart = myfilesizepart + myfilesizeparttmp

		" set case sensitivity for specific load
		let k = strpart(myfile, strlen(myfile) - 5, 1)
		if k == 1 || k == 3
			syntax case ignore
		elseif k == 2 || k == 4
			syntax case match
		endif

		" echo progress
		call ProgressBar(myfilesizepart, "    Loading dictionaries... ", "=", 0)

		" source dictionary of choice
		call Cream_source(myfile)

		let i = i + 1

	endwhile

endfunction

" Cream_SpchkNxt() {{{1
function! Cream_SpchkNxt()
" Next spelling error
	let errid = synIDtrans(hlID("BadWord"))
	let lastline = line("$")
	let curcol = 0
	normal w

	" skip words until we find next error
	while synIDtrans(synID(line("."),col("."),1)) != errid
		normal w
		if line(".") == lastline
			let prvcol = curcol
			let curcol = col(".")
			if curcol == prvcol
				break
			endif
		endif
	endwhile
endfunction

" Cream_SpchkPrv() {{{1
function! Cream_SpchkPrv()
" Previous spelling error
	let errid = synIDtrans(hlID("BadWord"))
	let curcol= 0
	normal b

	" skip words until we find previous error
	while synIDtrans(synID(line("."),col("."),1)) != errid
		normal b
		if line(".") == 1
			let prvcol=curcol
			let curcol=col(".")
			if curcol == prvcol
				break
			endif
		endif
	endwhile
endfunction

" Cream_SpchkSave() {{{1
function! Cream_SpchkSave(newword)
" save a new word to a user dictionary ($CREAM_SPELLDICTS/cream-spell-dict-usr_?.vim). 
" * must verify valid characters prior to using

	" determine which dictionary it goes in

	" case sensitive?
	" no uppercase characters found
	if     match(a:newword, "[A-Z]") == -1
		let case = "no"
	" no lowercase characters found
	elseif match(a:newword, "[a-z]") == -1
		let case = "yes"
	" mixed--verify with dialog
	else
		let myreturn = confirm("Is this word, \"" . a:newword . "\", case sensitive?", "&Yes\n&\No", 1, "Info")
		if myreturn == 1
			let case = "yes"
		elseif myreturn == 2
			let case = "no"
		else
			" use must have hit esc... quit.
			return -1
		endif
	endif

	" multi-word?
	if   match(a:newword, " ") != -1
	 " || match(a:newword, "-") != -1
	 " || match(a:newword, "&") != -1
		let multword = "yes"
	else
		let multword = "no"
	endif

	" determine correct dictionary part
	" * white space == 3 or 4
	" * mixed case, all upper == 1 or 3
	if     multword == "no" && case == "no"
		let myext = "_1"
	elseif multword == "no" && case == "yes"
		let myext = "_2"
	elseif multword == "yes" && case == "no"
		let myext = "_3"
	elseif multword == "yes" && case == "yes"
		let myext = "_4"
	endif

	" set path
	let myfile = g:cream_user . "spelldicts/cream-spell-dict-usr" . myext . ".vim"

	" open correct file (use split to preserve existing splits)
	execute "silent new " . myfile

	" insert syntax string plus word line at end of document
	if multword == "yes"
		" final form:  syntax match GoodWord "\<\(naive\|passe\)\>"
		let @x = "syntax match GoodWord \"\\\<\\(" . a:newword . "\\)\\>\""
	else
		let @x = "syntax keyword GoodWord transparent " . a:newword
	endif
	$put x

	" write and quit
	silent! write
	silent! bwipeout!

	" now execute the same statement to add it to the current
	" dictionary in memory (much faster than toggling off/on!
	execute @x

	"*** BROKEN (turns everything off)
	"" remove BadWord highlight from current environment
	"if     multword == "no" && case == "no"
	"	syntax case ignore
	"	execute "syntax keyword GoodWord transparent " . a:newword
	"elseif multword == "no" && case == "yes"
	"	syntax case match
	"	execute "syntax keyword GoodWord transparent " . a:newword
	"elseif multword == "yes" && case == "no"
	"	syntax case ignore
	"	execute "syntax match GoodWord \"\\\<" . a:newword . "\>\""
	"elseif multword == "yes" && case == "yes"
	"	syntax case match
	"	execute "syntax match GoodWord \"\\\<" . a:newword . "\>\""
	"endif
	"***

endfunction

" Cream_SpchkAlternate() {{{1
"function! <SID>Cream_SpchkAlternate(wrd)
"*** This routine uses agrep, not available on typical Windows systems!
" * suggest words that are close in spelling
"
"    if exists("g:esc_alternate")
"        " re-use wordlist in bottom window
"        execute "normal \<c-w>bG0DAAlternate<" . a:wrd . ">: \<Esc>"
"
"	 " *** don't forget to fix g:cream_user stuff! ***
"    elseif filereadable(expand("g:cream_spelldicts/cream-spell-dict-" . g:CREAM_SPELL_LANG . ".wordlist"))
"        " read in previously generated <engspchk.wordlist>
"        let g:esc_alternate= 1
"        bo 1new
"        setlocal lz
"        setlocal winheight=1
"        silent execute "normal GoAlternate<" . a:wrd . ">: \<Esc>"
"    else
"        " generate <engspchk.wordlist> from <engspchk.vim>
"        let g:esc_alternate= 1
"        echo "Building <cream-spell-dict-" . g:CREAM_SPELL_LANG . ".wordlist>..."
"        bo 1new
"        setlocal lz
"        setlocal winheight=1
"        execute "silent 0r $CREAM_SPELLDICTS/cream-spell-dict-" . g:CREAM_SPELL_LANG . ".vim"
"        silent 1,/The Dictionary/+1d
"        silent /The Raison D/-1,$d
"        %s/^syntax keyword GoodWord transparent\t//
"        silent execute "%s/\\s\\+/\<CR>/g"
"        execute "w! $CREAM_SPELLDICTS/cream-spell-dict-" . g:CREAM_SPELL_LANG . ".wordlist"
"        silent %d
"        silent execute "normal $oAlternate<" . a:wrd . ">: \<Esc>"
"    endif

"    nnoremap <buffer> <CR> <C-W>khea#<Esc>bde<C-W>blbye<C-W>kPlxb
"    cnoremap <buffer> q :unlet g:esc_alternate<CR>:q!<CR>
"    setlocal nomodified nowrap ic

"    " let's add a wee bit of color...
"    syntax match altLeader "^Alternate"
"    syntax match altAngle "[<>]"
"    highlight def link altLeader Statement
"    highlight def link altAngle Delimiter

"    " Build patterns based on permutations of up to first 3 letters
"    execute "normal \<c-w>b"
"    "+++ Cream: DUDE! we don't have agrep on Windows!
"    if strlen(a:wrd) <= 4
"        execute "read! agrep -2 -i -w -S2 " . a:wrd . " " . expand("$CREAM_SPELLDICTS/cream-spell-dict-" . g:CREAM_SPELL_LANG . ".wordlist")
"    else
"        execute "read! agrep -2 -i -w -S2 " . a:wrd . " " . expand("$CREAM_SPELLDICTS/cream-spell-dict-" . g:CREAM_SPELL_LANG . ".wordlist")
"    endif
"    "+++
"    %j
"    normal 04w
"    setlocal nomod

"endfunction

" Select dictionary language(s)
" * Allow choice of no extra dictionary 
" (default), single dictionary or multiple
" * empty if default
" * Provide dialogs to select/toggle depending on if multiple allowed


" Cream_spell_dict_avail() {{{1
function! Cream_spell_dict_avail()
" return comma-separated list of three-character abbreviations for
" available language dictionaries
" * The entire dictionary is said to be available even if only one
"   part of a multi-part exists.
	let myavail = ""
	" Dutch
	let mylang = glob(g:cream_spelldicts . "cream-spell-dict-dut*.vim")
	let i = 0
	let max = MvNumberOfElements(mylang, "\n")
	while i < max
		let myfile = MvElementAt(mylang, "\n", i)
		if filereadable(myfile) > 0
			let myavail = myavail . "dut,"
			break
		endif
		let i = i + 1
	endwhile
	" English
	let mylang = glob(g:cream_spelldicts . "cream-spell-dict-eng*.vim")
	let i = 0
	let max = MvNumberOfElements(mylang, "\n")
	while i < max
		let myfile = MvElementAt(mylang, "\n", i)
		if filereadable(myfile) > 0
			let myavail = myavail . "eng,"
			break
		endif
		let i = i + 1
	endwhile
	" French
	let mylang = glob(g:cream_spelldicts . "cream-spell-dict-fre*.vim")
	let i = 0
	let max = MvNumberOfElements(mylang, "\n")
	while i < max
		let myfile = MvElementAt(mylang, "\n", i)
		if filereadable(myfile) > 0
			let myavail = myavail . "fre,"
			break
		endif
		let i = i + 1
	endwhile
	" German
	let mylang = glob(g:cream_spelldicts . "cream-spell-dict-ger*.vim")
	let i = 0
	let max = MvNumberOfElements(mylang, "\n")
	while i < max
		let myfile = MvElementAt(mylang, "\n", i)
		if filereadable(myfile) > 0
			let myavail = myavail . "ger,"
			break
		endif
		let i = i + 1
	endwhile
	" Spanish
	let mylang = glob(g:cream_spelldicts . "cream-spell-dict-spa*.vim")
	let i = 0
	let max = MvNumberOfElements(mylang, "\n")
	while i < max
		let myfile = MvElementAt(mylang, "\n", i)
		if filereadable(myfile) > 0
			let myavail = myavail . "spa,"
			break
		endif
		let i = i + 1
	endwhile
	" others (not yet multi-part)
	if filereadable(g:cream_spelldicts. "cream-spell-dict-fre.vim") > 0
		let myavail = myavail . "fre,"
	endif

	if myavail != ""
		return myavail
	else
		return -1
	endif
endfunction


" Cream_spell_dialect() {{{1
function! Cream_spell_dialect(lang)
" return dialect(s) list (of a:lang) from dictionaries available
" * a:lang is the language code string including lang, size and dialects

	" establish three-char language code
	let mylang = MvElementAt(a:lang, ",", 0)

	" get list of available dialects
	let avail = Cream_spell_dialect_avail(mylang)

	" sort available into the same order as selection (reverse order)
	if mylang == "eng"
		let temp = ""
		if MvContainsElement(avail, ",", "ca") == 1
			let temp = MvInsertElementAt(avail, ",", "ca", 0)
		endif
		if MvContainsElement(avail, ",", "br") == 1
			let temp = MvInsertElementAt(avail, ",", "br", 0)
		endif
		if MvContainsElement(avail, ",", "us") == 1
			let temp = MvInsertElementAt(avail, ",", "us", 0)
		endif
		let avail = temp
	endif

	let myreturn = ""

	" current dialects equal lang string minus first two entries
	let mycnt = MvNumberOfElements(a:lang, ",")
	if     mycnt > 2
		let mydialects_ = MvRemoveElementAt(a:lang, ",", 0)
		let mydialects_ = MvRemoveElementAt(mydialects_, ",", 0)
	elseif mycnt > 1
		let mydialects_ = MvRemoveElementAt(a:lang, ",", 0)
	elseif mycnt == 1
		" protect from none existing
		let mydialects_ = ""
	endif

""*** DEBUG:
"call confirm(
"	\"DEBUG: dialect removed in Cream_spell_dialect_dialog()... \n\n" .
"	\" mylang        = " . mylang . "\n" .
"	\" mydialects_   = " . mydialects_ . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***

	while myreturn == ""

		" english
		if mylang == "eng"

			" set up dialog buttons
			let buttons = ""
			if MvContainsElement(avail, ",", "us") == 1
				let buttons = buttons . "&U.S.\n"
			endif
			if MvContainsElement(avail, ",", "br") == 1
				let buttons = buttons . "&British\n"
			endif
			if MvContainsElement(avail, ",", "ca") == 1
				let buttons = buttons . "&Canadian\n"
			endif
			let buttons = buttons . "&Ok"

			" prepare dialog (allow multiple dialects)
			let msg = "Please toggle the active spelling dictionary dialects...\n\n\n"
			if MvContainsElement(avail, ",", "us") == 1
				if MvContainsElement(mydialects_, ",", "us") == 1
					let msg = msg . "               [X] Yes   [_] No          U.S.\n"
				else
					let msg = msg . "               [_] Yes   [X] No          U.S.\n"
				endif
			endif
			if MvContainsElement(avail, ",", "br") == 1
				if MvContainsElement(mydialects_, ",", "br") == 1
					let msg = msg . "               [X] Yes   [_] No          British\n"
				else
					let msg = msg . "               [_] Yes   [X] No          British\n"
				endif
			endif
			if MvContainsElement(avail, ",", "ca") == 1
				if MvContainsElement(mydialects_, ",", "ca") == 1
					let msg = msg . "               [X] Yes   [_] No          Canadian\n"
				else
					let msg = msg . "               [_] Yes   [X] No          Canadian\n"
				endif
			endif
			let msg = msg . "\n"
		else
			return
		endif

		let n = ""
		let n = confirm(msg, buttons, MvNumberOfElements(avail, ",") + 1, "Info")

		" evaluate return
		let maxn = MvNumberOfElements(avail, ",")
		if n < maxn

			" evaluate each possibility separately
			let i = 1
			let maxi = MvNumberOfElements(avail, ",")
			while i < maxi
				if n == i
					" position is NOT the language
					" get chosen dialect
					let mydialect = MvElementAt(avail, ",", i - 1)
					" already selected?
					if MvContainsElement(mydialects_, ",", mydialect) == 1
						" remove if already selected
						let mydialects_ = MvRemoveElement(mydialects_, ",", mydialect)
""*** DEBUG:
"call confirm(
"	\"DEBUG: dialect removed in Cream_spell_dialect_dialog()... \n\n" .
"	\" i             = " . i . "\n" .
"	\" mydialect     = " . mydialect . "\n" .
"	\" mydialects_   = " . mydialects_ . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
					else
						" add if not already selected
						let mydialects_ = MvInsertElementAt(mydialects_, ",", mydialect, 0)
""*** DEBUG:
"call confirm(
"	\"DEBUG: dialect chosen in Cream_spell_dialect_dialog()... \n\n" .
"	\" i             = " . i . "\n" .
"	\" mydialect     = " . mydialect . "\n" .
"	\" mydialects_   = " . mydialects_ . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
					endif

				endif
				let i = i + 1

			endwhile
			
		else
			" finished
			let myreturn = 1
		endif

	endwhile

	return mydialects_

endfunction

" Cream_spell_dialect_avail() {{{1
function! Cream_spell_dialect_avail(lang)
" return comma-separated list of dialects available for a language

		let myavail = ""
		" directory the language (*** ASSUMPTION: "_1" is representative of available! ***)
		let mylang = Cream_getfilelist(g:cream_spelldicts . "cream-spell-dict-" . a:lang . "*_1.vim")
		" find/parse out the available dialects of each filename
		let i = 0
		let max = MvNumberOfElements(mylang, "\n")
		while i < max

			let myfile = MvElementAt(mylang, "\n", i)
			" get two digits in filename immediately preceeding "_#.vim"
			let mydialect = strpart(myfile, strlen(myfile) - 8, 2)

			" will contain a "-" if not
			if match(mydialect, "-") == -1
				" add if doesn't already exist
				if match(myavail, mydialect) == -1
					let myavail = MvAddElement(myavail, ",", mydialect)
				endif
			endif
""*** DEBUG:
"call confirm(
"	\"DEBUG: \n\n" .
"	\"Cream_spell_dialect_avail() \n" .
"	\" myfile        = " . myfile . "\n" .
"	\" mydialect     = " . mydialect . "\n" .
"	\" myavail       = " . myavail . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
			let i = i + 1

		endwhile
""*** DEBUG:
"call confirm(
"	\"DEBUG: \n\n" .
"	\"Cream_spell_dialect_avail() \n" .
"	\" myavail     = " . myavail . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
		return myavail

endfunction

" Cream_spell_lang() {{{1
function! Cream_spell_lang()

	" save guioptions environment
	let myguioptions = &guioptions
	" do not use a vertical layout for dialog buttons
	set guioptions-=v

	" verify initialization
	if !exists("g:CREAM_SPELL_MULTIDICT")
		let g:CREAM_SPELL_MULTIDICT = 0
	endif

	" verify use of additional language dictionaries
	if g:CREAM_SPELL_MULTIDICT == 0
		" OBSOLETE: don't verify, just do it
		"let n = confirm("Would you like to turn on the optional language dictionaries?\n", "&Yes\n&No", 1, "Info")
		"if n == 1
			let g:CREAM_SPELL_MULTIDICT = 1
		"else
		"	return -1
		"endif
	endif

	" get options
	let valid = '0'
	while valid == '0'
		" get find, replace and path/filename info
		let myreturn = Cream_spell_lang_dialog()
		" if quit code returned
		if myreturn == '1' || myreturn == '-1'
			break
		endif

""*** DEBUG:
"call confirm(
"	\"DEBUG: in Cream_spell_lang()\n\n" .
"	\"g:CREAM_SPELL_LANG      = " . g:CREAM_SPELL_LANG . "\n" .
"	\"g:CREAM_SPELL_MULTIDICT = " . g:CREAM_SPELL_MULTIDICT . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
	endwhile

	" refresh if spell check on
	if exists("b:cream_spell") && b:cream_spell == "1"
		syntax enable
		call Cream_spell()
	endif

	" refresh autocorrect
	call Cream_autocorrect_init()

	" restore guioptions
	execute "set guioptions=" . myguioptions

endfunction

" Cream_spell_lang_dialog() {{{1
function! Cream_spell_lang_dialog()
" Returns 1 if ok, -1 on cancel

	" get list of available dictionary languages
	let avail = Cream_spell_dict_avail()
	if avail == -1
		call confirm(
			\"You do not have multiple language dictionaries available to reference.\n" .
			\"\n" .
			\"Please download additional optional language dictionaries from http://cream.sourceforge.net !\n" .
			\"\n", "&Ok", 1, "Info")
		let g:CREAM_SPELL_MULTIDICT = 0
		let g:CREAM_SPELL_LANG = ""
		return -1
	endif

	" sort available into the same order as selection (reverse order)
	let temp = ""
	if MvContainsElement(avail, ",", "spa") == 1
		let temp = MvInsertElementAt(temp, ",", "spa", 0)
	endif
	if MvContainsElement(avail, ",", "ger") == 1
		let temp = MvInsertElementAt(temp, ",", "ger", 0)
	endif
	if MvContainsElement(avail, ",", "fre") == 1
		let temp = MvInsertElementAt(temp, ",", "fre", 0)
	endif
	if MvContainsElement(avail, ",", "eng") == 1
		let temp = MvInsertElementAt(temp, ",", "eng", 0)
	endif
	if MvContainsElement(avail, ",", "dut") == 1
		let temp = MvInsertElementAt(temp, ",", "dut", 0)
	endif
	let avail = temp

	" if available, add to dialog
	let buttons = ""
	if MvContainsElement(avail, ",", "dut") == 1
		let buttons = buttons . "&Dutch\n"
	endif
	if MvContainsElement(avail, ",", "eng") == 1
		let buttons = buttons . "&English\n"
	endif
	if MvContainsElement(avail, ",", "fre") == 1
		let buttons = buttons . "&French\n"
	endif
	if MvContainsElement(avail, ",", "ger") == 1
		let buttons = buttons . "&German\n"
	endif
	if MvContainsElement(avail, ",", "spa") == 1
		let buttons = buttons . "&Spanish\n"
	endif

	let n = 0

	" if multidict status to use default dictionary, abort
	if g:CREAM_SPELL_MULTIDICT == 0
		" user hasn't selected optional dictionary referencing, quit.
		let g:CREAM_SPELL_LANG = ""
		return 2
	endif

	"" if coming up from multidict==0, will be empty; set default
	"if g:CREAM_SPELL_LANG == ""
	"	let g:CREAM_SPELL_LANG = "eng,s"
	"endif

	" set up dialog box based on multidict status
	if g:CREAM_SPELL_MULTIDICT == 1

		let msg = "Please select the language of the spelling dictionary...\n\n"

		let buttons = buttons . "&Cancel"

	elseif g:CREAM_SPELL_MULTIDICT == 2

		let msg = "Please toggle the active spelling dictionaries...\n"
		let msg = msg . "\n"
		let msg = msg . "\n"
		if MvContainsElement(avail, ",", "dut") == 1
			if match(g:CREAM_SPELL_LANG, "dut") >= 0
				let msg = msg . "               [X] Yes   [_] No          Dutch\n"
			else
				let msg = msg . "               [_] Yes   [X] No          Dutch\n"
			endif
		endif
		if MvContainsElement(avail, ",", "eng") == 1
			if match(g:CREAM_SPELL_LANG, "eng") >= 0
				let msg = msg . "               [X] Yes   [_] No          English\n"
			else
				let msg = msg . "               [_] Yes   [X] No          English\n"
			endif
		endif
		if MvContainsElement(avail, ",", "fre") == 1
			if match(g:CREAM_SPELL_LANG, "fre") >= 0
				let msg = msg . "               [X] Yes   [_] No          French\n"
			else
				let msg = msg . "               [_] Yes   [X] No          French\n"
			endif
		endif
		if MvContainsElement(avail, ",", "ger") == 1
			if match(g:CREAM_SPELL_LANG, "ger") >= 0
				let msg = msg . "               [X] Yes   [_] No          German\n"
			else
				let msg = msg . "               [_] Yes   [X] No          German\n"
			endif
		endif
		if MvContainsElement(avail, ",", "spa") == 1
			if match(g:CREAM_SPELL_LANG, "spa") >= 0
				let msg = msg . "               [X] Yes   [_] No          Spanish\n"
			else
				let msg = msg . "               [_] Yes   [X] No          Spanish\n"
			endif
		endif
		let msg = msg . "\n"

		let buttons = buttons . "&Ok"

	endif

	" display dialog (sets mylang)
	let n = confirm(msg, buttons, MvNumberOfElements(avail, ",") + 1, "Info")
	" iterate through potential choices
	let max = MvNumberOfElements(avail, ",")
	if n <= max
		let i = 1
		let max1 = MvNumberOfElements(avail, ",")
		while i <= max1
			" act upon one selected
			if n == i
				let mylang = MvElementAt(avail, ",", i - 1)
			endif
			let i = i + 1
		endwhile
	else
		" user canceled, abort
		return -1
	endif

	" get position of language in existing string
	let mylangpos = Cream_spell_getlangstrpos(mylang)

	" only process language specifics *if* being added (usually, but
	" not when multi-dict toggling off)
	if g:CREAM_SPELL_MULTIDICT == 1 ||
		\g:CREAM_SPELL_MULTIDICT == 2 && mylangpos == -1

		" DONT' CONDITION, do for all languages
		"" select individual language options
		"if mylang == "eng"

			" get size (will return at least 1 except on error/cancel)
			let mysize = Cream_spell_size(mylang)
			if mysize == -1
				call confirm("Using default size...", "&Ok", 1, "Info")
				let mysize = "s"
			endif

			" get dialects
			" useful if language isn't changing but dialects/size are
			let mylangstr = Cream_spell_getlangstr(mylang)
			let mydialect = Cream_spell_dialect(mylangstr)

		"endif

		" cat full individual language string
		" we return something other than "" (cancel) or "0" (doesn't exist)
		if strlen(mydialect) > 1
			let mynewlang = mylang . "," . mysize . "," . mydialect
		else
			let mynewlang = mylang . "," . mysize
		endif

	endif

	" set dictionary based on multidict status
	if g:CREAM_SPELL_MULTIDICT == 1
		let g:CREAM_SPELL_LANG = ""
		let g:CREAM_SPELL_LANG = MvInsertElementAt(g:CREAM_SPELL_LANG, ";", mynewlang, 0)
""*** DEBUG:
"call confirm(
"	\"DEBUG: new lang string in Cream_spell_lang_dialog(), 1... \n\n" .
"	\" g:CREAM_SPELL_LANG = " . g:CREAM_SPELL_LANG . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
		return 1
	elseif g:CREAM_SPELL_MULTIDICT == 2
		" if didn't exist, add
		if mylangpos == -1
			let g:CREAM_SPELL_LANG = MvInsertElementAt(g:CREAM_SPELL_LANG, ";", mynewlang, 0)
		" else remove
		else
			let g:CREAM_SPELL_LANG = MvRemoveElementAt(g:CREAM_SPELL_LANG, ";", mylangpos)
		endif
""*** DEBUG:
"call confirm(
"	\"DEBUG: new lang string in Cream_spell_lang_dialog(), 2... \n\n" .
"	\" g:CREAM_SPELL_LANG = " . g:CREAM_SPELL_LANG . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
		return 2
	endif

endfunction

" Cream_spell_size() {{{1
function! Cream_spell_size(lang)
" return size selected, -1 if none selected

	let temp = ""
	" get list of available sizes in language
	let avail = Cream_spell_size_avail(a:lang)

	" setup dialog
	" sort available into the same order as selection (reverse order)
	if MvContainsElement(avail, ",", "x") == 1
		let temp = MvInsertElementAt(temp, ",", "x", 0)
	endif
	if MvContainsElement(avail, ",", "l") == 1
		let temp = MvInsertElementAt(temp, ",", "l", 0)
	endif
	if MvContainsElement(avail, ",", "m") == 1
		let temp = MvInsertElementAt(temp, ",", "m", 0)
	endif
	if MvContainsElement(avail, ",", "s") == 1
		let temp = MvInsertElementAt(temp, ",", "s", 0)
	endif
	let avail = temp

	let mylang = Cream_spell_getlangstr(a:lang)
	if mylang == -1
		" start with this language if empty
		let mylang = a:lang
	endif

	" if available, add to dialog
	let buttons = ""
	if MvContainsElement(avail, ",", "s") == 1
		let buttons = buttons . "&Small (default)\n"
	endif
	if MvContainsElement(avail, ",", "m") == 1
		let buttons = buttons . "&Medium\n"
	endif
	if MvContainsElement(avail, ",", "l") == 1
		let buttons = buttons . "&Large\n"
	endif
	if MvContainsElement(avail, ",", "x") == 1
		let buttons = buttons . "&Huge\n"
	endif

	" prepare dialog (radio button-like -- never allow multiple dialects)
	let buttons = buttons . "&Cancel"

	" dialog
	let n = confirm(
		\"Please select the prefered dictionary size. Please note:\n" .
		\" * The larger the dictionary, the slower the error highlighting.\n" .
		\" * Huge size indicates a dictionary which includes such rare words\n" .
		\"    that common mis-spellings could be passed (such as \"ort\" a common\n" .
		\"    mis-spelling of \"or\".\n" .
		\"\n", buttons, MvNumberOfElements(avail, ",") + 1, "Info")
	let max = MvNumberOfElements(avail, ",")
	if n <= max
		let i = 1
		let max1 = MvNumberOfElements(avail, ",")
		while i <= max1
			if n == i
				let mysize = MvElementAt(avail, ",", i - 1)
			endif
			let i = i + 1
		endwhile
""*** DEBUG:
"call confirm(
"	\"DEBUG: in Cream_spell_size()... \n\n" .
"	\" mysize = " . mysize . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
		return mysize
	else
		return -1
	endif

endfunction

" Cream_spell_size_avail() {{{1
function! Cream_spell_size_avail(lang)
" return comma-separated list of dict sizes available for a language

		let myavail = ""
		" directory the language (*** ASSUMPTION: "_1" is representative of available! ***)
		let mylang = Cream_getfilelist(g:cream_spelldicts . "cream-spell-dict-" . a:lang . "*_1.vim")
		" find/parse out the available sizes of each filename
		let i = 0
		let max = MvNumberOfElements(mylang, "\n")
		while i < max

			let myfile = MvElementAt(mylang, "\n", i)
			" get single char in filename one char following a:lang
			" find pos of lang
			let mylangpos = stridx(fnamemodify(myfile, ":t:r"), a:lang)
			" get char that pos + lang's length, + 1 separator width, 1 char wide
			let mysizepos = mylangpos + strlen(a:lang) + 1
			let mysize = strpart(fnamemodify(myfile, ":t:r"), mysizepos, 1)
""*** DEBUG:
"let myreturn = confirm(
"	\"DEBUG: \n\n" .
"	\"Cream_spell_size_avail() \n" .
"	\" myfile        = " . myfile . "\n" .
"	\" mylangpos     = " . mylangpos . "\n" .
"	\" mysizepos     = " . mysizepos . "\n" .
"	\" mysize        = " . mysize . "\n" .
"	\"\n", "&Ok", 1, "Info")
"if myreturn != 1
"	return
"endif
""***
			" add if doesn't already exist
			if match(myavail, mysize) == -1
				let myavail = MvAddElement(myavail, ",", mysize)
			endif
""*** DEBUG:
"call confirm(
"	\"DEBUG: \n\n" .
"	\"Cream_spell_size_avail() \n" .
"	\" myavail       = " . myavail . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
			let i = i + 1

		endwhile
""*** DEBUG:
"call confirm(
"	\"DEBUG: \n\n" .
"	\"Cream_spell_size_avail() \n" .
"	\" myavail     = " . myavail . "\n" .
"	\"\n", "&Ok", 1, "Info")
""***
		return myavail

endfunction

" Cream_spell_getlangstr() {{{1
function! Cream_spell_getlangstr(lang)
" return a specific language's elements (mylang) out of g:CREAM_SPELL_LANG
" return language's code if doesn't exist
	" iterate through language array, 
	let i = 0
	let mylang = ""
	let max = MvNumberOfElements(g:CREAM_SPELL_LANG, ";")
	while i <= max
		let mylangstr = MvElementAt(g:CREAM_SPELL_LANG, ";", i)
		" if first element matches language
		if MvElementAt(mylangstr, ",", 0) == a:lang
			return mylangstr
		endif
		let i = i + 1
	endwhile
	return a:lang
endfunction

" Cream_spell_getlangstrpos() {{{1
function! Cream_spell_getlangstrpos(lang)
" return language's position in array; 0 based; -1 if doesn't exist
	" iterate through language array
	let i = 0
	let max = MvNumberOfElements(g:CREAM_SPELL_LANG, ";")
	while i <= max
		let mylang = MvElementAt(g:CREAM_SPELL_LANG, ";", i)
		" if first element matches language
		if MvElementAt(mylang, ",", 0) == a:lang
			return i
		endif
		let i = i + 1
	endwhile
	return -1
endfunction

" Cream_spell_options() {{{1
function! Cream_spell_options()
" prompt for spell check options (default, single or multiple dictionaries)

	" save guioptions environment
	let myguioptions = &guioptions
	" do not use a vertical layout for dialog buttons
	set guioptions-=v

	let avail = Cream_spell_dict_avail()
	" if optional dictionary(s) not available, quit
	let max = MvNumberOfElements(avail, ",")
	if max == 0
		call confirm(
			\"You do not have any language dictionaries available to reference.\n" .
			\"\n" .
			\"Please download optional language dictionaries from http://cream.sourceforge.net !\n" .
			\"\n", "&Ok", 1, "Info")
		if exists("g:CREAM_SPELL_LANG")
			let g:CREAM_SPELL_LANG = ""
		endif
	else
		" get options
		let mystatus = g:CREAM_SPELL_MULTIDICT
		let valid = 0
		while valid == 0
			" get find, replace and path/filename info
			let myreturn = Cream_spell_options_dialog(mystatus)
			" if enter quit code returned
			if myreturn == -1
				let g:CREAM_SPELL_MULTIDICT = mystatus
				break
			endif
			" increment
			if mystatus == 0
				let mystatus = 1
			elseif mystatus == 1
				let mystatus = 2
			elseif mystatus == 2
				let mystatus = 0
			endif
		endwhile
	endif

	" condition environment based on g:CREAM_SPELL_MULTIDICT
	if g:CREAM_SPELL_MULTIDICT == 0
		let g:CREAM_SPELL_LANG = ""
	elseif g:CREAM_SPELL_MULTIDICT == 1
		" strip languages above first 
		" (We rebuild to ensure single element is properly terminated.
		" Cream prior to v.13 did not use a multi-dimensional array,
		" and separated language elements with commas. By re-building,
		" we can clean up the existing variable state.)

		" get first element
		let tempstr = MvElementAt(g:CREAM_SPELL_LANG, ";", 0)
		" clear variable
		let g:CREAM_SPELL_LANG = ""
		" re-define array as that element
		let g:CREAM_SPELL_LANG = MvInsertElementAt(g:CREAM_SPELL_LANG, ";", tempstr, 0)
	elseif g:CREAM_SPELL_MULTIDICT == 2
		" can be empty or full
	endif

	" select dictionary if none present and dictionaries selected
	if g:CREAM_SPELL_LANG == "" && g:CREAM_SPELL_MULTIDICT > 0
		call Cream_spell_lang()
	endif

	" refresh if spell check on
	if exists("b:cream_spell") && b:cream_spell == "1"
		syntax enable
		" reload dictionaries
		call Cream_spell()
	endif

	" restore guioptions
	execute "set guioptions=" . myguioptions

endfunction

" Cream_spell_options_dialog() {{{1
function! Cream_spell_options_dialog(status)

	let mychoice = 0
	let msg = "Options:\n"
	let msg = msg . "\n"
	let msg = msg . "\n"
	let msg = msg . "    Dictionary referencing:\n"
	if a:status == 0
		let msg = msg . "               [X] Basic\n"
		let msg = msg . "               [_] Single Language\n"
		let msg = msg . "               [_] Multiple Language\n"
	elseif a:status == 1
		let msg = msg . "               [_] Basic\n"
		let msg = msg . "               [X] Single Language\n"
		let msg = msg . "               [_] Multiple Language\n"
	elseif a:status == 2
		let msg = msg . "               [_] Basic\n"
		let msg = msg . "               [_] Single Language\n"
		let msg = msg . "               [X] Multiple Language\n"
	endif
	let msg = msg . "\n"
	let mychoice = confirm(msg, "Dictionary Referencing (toggle)\n&Ok", 1)
	if mychoice == 1
		" increment and return
		if a:status == 0
			return 1
		elseif a:status == 1
			return 2
		elseif a:status == 2
			return 0
		endif
	else
		return -1
	endif

endfunction
" 1}}}
" vim:foldmethod=marker
